"""Console script for open_dev."""
import textwrap
import time

import rich_click as click
from rich.progress import track

from open_dev.constants import HEADER
from open_dev.src.open_dev_repo import OpenDevRepo

click.rich_click.USE_MARKDOWN = True


def fstr(string):
    """Formatr strings from the interface."""
    return string.split(" ")


@click.group()
def repo():
    """Tooling to enable devs to buidl quick."""


@click.command()
def create():
    """Creates a new repository."""
    click.echo('Hello there')


@click.command()
def sync():
    """Creates a new repository."""
    click.echo('Synchronise state so remote and local are aligned')


@click.command()
def info():
    """Retrieves infromation about the current repo."""

    current_repo = OpenDevRepo()
    repo_info = textwrap.dedent(
        f"""
    Currently managing:

        name: \t\t{current_repo.name}
        hash: \t\t{current_repo.current_head[:8]}
        branch: \t\t{current_repo.branch}
        status: \t\t{current_repo.status}


    """
    )
    click.echo(repo_info)


@click.command()
def watch():
    """Creates a new repository."""
    click.echo('Hello there')


@click.group()
def deps():
    """Manage Dependencies."""
    click.echo('Lets sort out those dependencies.')


@click.group()
def test():
    """Manage Tests."""
    click.echo('Lets run some tests.')


@click.command()
def run_all():
    """Test all."""
    click.echo('Starting Tests.')
    for _ in track(range(20), description="Processing..."):
        time.sleep(0.05)
    click.echo(
        'Success!',
        color="green",
    )


@click.group()
def tasks():
    """Manage Tasks."""


@click.command()
def install():
    """Synchronise dependencies."""
    click.echo(
        """
    Install dependencies with deps with
    ```bash
    pip install -e . --user
    ````
    """
    )


click.echo(textwrap.dedent(HEADER))


@click.group()
def main():
    """Oh dev tooling to speed up development."""


deps.add_command(install)
repo.add_command(create)
repo.add_command(sync)
repo.add_command(info)
test.add_command(run_all)

for group in [repo, deps, test]:
    main.add_command(group)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['open_dev', 'open_dev.src', 'tests', 'tests.app']

package_data = \
{'': ['*']}

install_requires = \
['GitPython>=3.1.27,<4.0.0',
 'click>=8.1.3,<9.0.0',
 'mkdocs-material-extensions>=1.0.3,<2.0.0',
 'rich-click>=1.5.2,<2.0.0',
 'yamllint>=1.27.1,<2.0.0']

extras_require = \
{':extra == "dev"': ['tox>=3.25.1,<4.0.0',
                     'pip>=22.2.2,<23.0.0',
                     'twine>=4.0.1,<5.0.0',
                     'pre-commit>=2.20.0,<3.0.0',
                     'toml>=0.10.2,<0.11.0',
                     'bump2version>=1.0.1,<2.0.0'],
 ':extra == "doc"': ['mkdocs>=1.3.1,<2.0.0',
                     'mkdocs-include-markdown-plugin>=3.6.1,<4.0.0',
                     'mkdocs-material>=8.4.0,<9.0.0',
                     'mkdocstrings>=0.19.0,<0.20.0',
                     'mkdocs-autorefs>=0.4.1,<0.5.0'],
 ':extra == "test"': ['black>=22.6.0,<23.0.0',
                      'isort>=5.10.1,<6.0.0',
                      'mypy>=0.971,<0.972',
                      'pytest-cov>=3.0.0,<4.0.0',
                      'pylama[all]>=8.4.1,<9.0.0']}

entry_points = \
{'console_scripts': ['odev = open_dev.cli:main']}

setup_kwargs = {
    'name': 'open-dev',
    'version': '0.1.5',
    'description': 'A collection of tooling to enable open source development..',
    'long_description': '# open_dev\n\n\n[![pypi](https://img.shields.io/pypi/v/open_dev.svg)](https://pypi.org/project/open_dev/)\n[![python](https://img.shields.io/pypi/pyversions/open_dev.svg)](https://pypi.org/project/open_dev/)\n[![Build Status](https://github.com/8ball030/open_dev/actions/workflows/dev.yml/badge.svg)](https://github.com/8ball030/open_dev/actions/workflows/dev.yml)\n[![codecov](https://codecov.io/gh/8ball030/open_dev/branch/main/graphs/badge.svg)](https://codecov.io/github/8ball030/open_dev)\n\n\nInstall Dependencies.\n\n(Poetry) Is used to managed the dependencies. (https://python-poetry.org/docs/#installation)\n# osx / linux / bashonwindows install instructions\n```bash\ncurl -sSL https://raw.githubusercontent.com/python-poetry/poetry/master/get-poetry.py | python -\n```\n```\n# windows install instructions\n```\n```\n(Invoke-WebRequest -Uri https://raw.githubusercontent.com/python-poetry/poetry/master/get-poetry.py -UseBasicParsing).Content | python `\n```\n\n\nA collection of tooling to enable open source development.\n\n\n* Documentation: <https://8ball030.github.io/open_dev>\n* GitHub: <https://github.com/8ball030/open_dev>\n* PyPI: <https://pypi.org/project/open_dev/>\n* Free software: Apache-2.0\n\n\n## Features\n\n* TODO\n\n## Credits\n\nThis package was created with [Cookiecutter](https://github.com/audreyr/cookiecutter) and the [waynerv/cookiecutter-pypackage](https://github.com/waynerv/cookiecutter-pypackage) project template.\n',
    'author': '8Baller',
    'author_email': '8ball030@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/8ball030/open_dev',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.7.2,<4.0',
}


setup(**setup_kwargs)



"""Test entities."""
from unittest.mock import patch

import pytest

from open_dev.src.open_dev_repo import OpenDevRepo, RepoStatus


@pytest.fixture(name='repo')
def repo_fixure():
    """Create a openDev repo."""
    with patch('open_dev.src.open_dev_repo.OpenDevRepo.branch', "main"):
        yield OpenDevRepo()


# @pytest.mark.skip()
def test_inits_repo(repo):
    """Creates the repo."""
    print(repo)


def test_gets_repo(repo):
    """Repo object retrieves the current head."""
    assert isinstance(repo.current_head, str)


@pytest.mark.skip()
def test_status(repo):
    """Repo object retrieves the current status."""
    assert isinstance(repo.status, RepoStatus)



"""Main module."""

import enum
from dataclasses import dataclass
from pathlib import Path

from git import Repo


class RepoStatus(enum.Enum):
    """Represets repository statuses."""

    LOCAL_DIRTY = enum.auto()
    LOCAL_AHEAD = enum.auto()
    REMOTE_AHEAD = enum.auto()
    SYNCED = enum.auto()
    WARNING = enum.auto()
    UNKNOWN = enum.auto()


@dataclass
class OpenDevRepo:
    """Class for Repos."""

    name: str = "open-dev"
    project_slug: str = "open_dev"
    cli_name = "o-dev"
    remote_path: str = "git@github.com:8ball030/open_dev.git"
    description: str = "A collection of tooling to enable open source development."
    local_path: Path = Path(".")
    detached: bool = False

    @property
    def git_repo(self) -> Repo:
        """Retrieve the repository."""
        assert self.local_path.exists()
        return Repo(self.local_path)

    @property
    def current_head(self) -> str:
        """Returns the current head."""
        return self.git_repo.head.object.hexsha

    @property
    def branch(self) -> str:
        """Returns the current head."""
        try:
            return self.git_repo.active_branch.name
        except TypeError:
            self.detached = True
            return self.current_head

    @property
    def status(self) -> RepoStatus:
        """Returns the current head."""
        num_ahead, num_behind = self.difference_to_target
        if self.git_repo.is_dirty():
            return RepoStatus.LOCAL_DIRTY
        elif not num_ahead and not num_behind:
            return RepoStatus.SYNCED
        elif num_ahead and not num_behind:
            return RepoStatus.LOCAL_AHEAD
        elif not num_ahead and num_behind:
            return RepoStatus.REMOTE_AHEAD
        elif num_ahead and num_behind:
            return RepoStatus.WARNING
        return RepoStatus.UNKNOWN

    @property
    def difference_to_target(self):
        """Checks the difference between current branch and remote."""
        target_branch = self.branch
        if self.detached:
            target_branch = "main"
        commits_diff = self.git_repo.git.rev_list('--left-right', '--count', f'{target_branch}@{{u}}')
        num_ahead, num_behind = commits_diff.split('\t')
        return int(num_ahead), int(num_behind)

    def __str__(self) -> str:
        return f"OpenDevRepo({self.remote_path})"

    def __repr__(self) -> str:
        return str(self)
